'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _babelPluginSyntaxFlow = require('babel-plugin-syntax-flow');

var _babelPluginSyntaxFlow2 = _interopRequireDefault(_babelPluginSyntaxFlow);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _babelTemplate = require('babel-template');

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var babylonOpts = { plugins: ['flow'] };

var typeDefineBuilder = (0, _babelTemplate2.default)('type NAME = VALUE', babylonOpts);

var KEY = Symbol('define-undefined-type');

function getPrefix(_ref) {
  var filename = _ref.opts.filename;
  var removePrefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

  var file = _path2.default.relative(_path2.default.join(process.cwd(), removePrefix), filename);
  var dirname = _path2.default.dirname(file);
  return dirname + '/';
}

var createDefineAST = function createDefineAST(name, value) {
  return typeDefineBuilder({
    NAME: t.identifier(name),
    VALUE: t.stringLiteral(value)
  });
};

exports.default = function () {
  return {
    inherits: _babelPluginSyntaxFlow2.default,
    name: 'define-undefined-type',
    pre: function pre(file) {
      if (!file.get(KEY)) {
        file.set(KEY, new Set());
      }
    },

    visitor: {
      TypeAlias: function TypeAlias(path, state) {
        var file = state.file,
            opts = state.opts;

        var definedTypes = file.get(KEY);

        definedTypes.add(path.get('id').node.name);

        function insertGenericType(nodePath) {
          var name = nodePath.get('id').node.name;
          if (!name || definedTypes.has(name)) {
            return;
          }

          var prefix = opts.usePrefix ? getPrefix(file, opts.removePrefix) : '';
          var value = prefix + name;
          var ast = createDefineAST(name, value);

          definedTypes.add(name);
          path.insertBefore(ast);
        }

        function insertUnionType(unionPath) {
          var typesPath = unionPath.get('types');
          var _iteratorNormalCompletion = true;
          var _didIteratorError = false;
          var _iteratorError = undefined;

          try {
            for (var _iterator = typesPath[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              var typePath = _step.value;

              insert(typePath);
            }
          } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
              }
            } finally {
              if (_didIteratorError) {
                throw _iteratorError;
              }
            }
          }
        }

        function insertObjectType(objPath) {
          var properties = objPath.get('properties');
          var _iteratorNormalCompletion2 = true;
          var _didIteratorError2 = false;
          var _iteratorError2 = undefined;

          try {
            for (var _iterator2 = properties[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
              var propPath = _step2.value;

              insert(propPath.get('value'));
            }
          } catch (err) {
            _didIteratorError2 = true;
            _iteratorError2 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
              }
            } finally {
              if (_didIteratorError2) {
                throw _iteratorError2;
              }
            }
          }
        }

        function insert(targetPath) {
          if (targetPath.isGenericTypeAnnotation()) {
            insertGenericType(targetPath);
          } else if (targetPath.isUnionTypeAnnotation()) {
            insertUnionType(targetPath);
          } else if (targetPath.isObjectTypeAnnotation()) {
            insertObjectType(targetPath);
          }
        }

        insert(path.get('right'));
      }
    }
  };
};